function [h,Di1,cp,Nat]=HBW_SPMP1_Trg_FFT(f,LN,tol,No,eex,icase,toln,Max,Maxp);
%HBW_OMP Hierarchical Block Wise Orthogonal Matching Pursuit (1D)
% 
%It creates an atomic decomposition of a 1 D signal divided into segments.
%At each iteration the algorithms selects
%i)the elements for approximating the segments composing the signal, using OMP criterion.
%ii)the hierarchical sequence in which the segments are approximated.
% You can choose a tolerance for the approximation or/and the total number of atoms to take in. 
% 
%  
%
% Inputs:
%   f       cell array f{n} is a vector with the signal for segment n 
%   Dx      dictionary of normalized atoms (w.r.t. Image rows)
%   tol     desired distance between f and its approximation H 
%   No      maximal number of atoms to choose, if the number of chosen atoms
%           equals to No, routine will stop (default numel(f))
%
% Outputs:
%   H       cell array with the approximation of f
%   Di      cell with the indices of selected atoms for the frame  w.r.t the original Dx
%   beta    cell array with the biorthogonal atoms  to the selected ones
%   c       cell array with coefficients of the atomic decomposition
%   Q       cell array with an ortho basis for the selected subspace
%   Nat     Total number of atoms (in case it is not No)
%  
%   
name='HBW-SPMP-FFT'; %name of routine
% get inputs and setup parameters
%!!!! modification of 2D
Lx=LN(1);
Nx=LN(2);
Ck=2*Nx/Lx*log2(2*Nx);
a=Nx/Lx;
if icase==4 a=a/2;end
%delta=1/(Lx*Ly);
delta=1;
NBx=numel(f);
N=NBx
%!!!end
zmax=1;%number of reorthogonalizations
beta=cell(N,1);
cc=cell(N,1);
cp=cell(N,1);
qq=cell(N,1);
Di1=cell(N,1);
Dic=cell(N,1);
Dis=cell(N,1);
Vx=cell(N,1);
Re=f;
tt=1:Lx;
%for ii=1:NBx
%B=Re{ii};
%nore(ii)=norm(B)^2;
%end
%
kq=zeros(N,1);
Cp=zeros(N,1);
%if (nargin<4) | (isempty(No)==1)  No=9*N;end
%if (nargin<3) | (isempty(tol)==1) tol=0.03*sum(nore);end; 
%
%atoms having smaller norm than tol1 are supposed be zero ones
tol1=1e-7; %1e-5
%threshold for coefficients
tol2=1e-11;   
Nat=0;
for nb=1:NBx
    B=Re{nb};
    nore(nb)=norm(B)^2; %from  the comment line 69 
%    cc{nb}=abs(Dx'*B); %modified from 2D
%    [max_c,b2]=max(cc{nb});
      [max_c,q,cscra]=AtSelFFT_1(B,Nx,eex,icase);
%     [max_c,q,cscra]=fast_chose_trig_atom(B',Nx,eex,icase);
    qq{nb}=q;  %modified from 2D
    Cp(nb)=max_c;
    cp{nb}=zeros(Nx,1);
    h{nb}=zeros(Lx,1);
end
%
%===============================
% Main algorithm 
%===============================
while Nat < No 
  %finding of maximal coefficient
    [Cpmax,qb]=max(Cp);
    q(1)=qq{qb};  %change from 2D
  %stopping criterion (coefficient)
    if Cpmax<tol2 
      fprintf('%s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',name,tol2);
      break;
    end
    kq(qb)=kq(qb)+1;
    k=kq(qb);
    Di1{qb}=[Di1{qb} q(1)]; 
         if icase==4
            if q(1)<=Nx/2 Dic{qb}=[Dic{qb} q(1)];
                else Dis{qb}=[Dis{qb} (q(1)-Nx/2)];
            end
         end
%
%   cscra=cc{qb}(q(1));
   AddAt=TrgAt(q(1),tt,a,icase);
   h_new=AddAt*cscra;
%  h_new=Dx(:,q(1))*cscra;
  cp{qb}(q(1))=cp{qb}(q(1)) + cscra;
% 
  h{qb}=h{qb}+h_new;
  Re{qb}=Re{qb}(:)-h_new;%residual of block qp
  Rein=Re{qb};
  hin=h{qb};
  id=numel(Di1{qb});
  if id <=Ck
  c=cp{qb}(Di1{qb});
  Vx{qb}=[Vx{qb} AddAt];
% Vx=Dx(:,Di1{qb});
%  [hout,c]=ProjMP(hin',Rein',Vx,c,toln,Max);
  [hout,c,ka]=ProjMPk(hin',Rein',Vx{qb},c,toln,Maxp);
  cp{qb}(Di1{qb})=c;
  else
  [hout,c,ka]=ProjMPTrgFFT(hin',Rein',LN,Di1{qb},Dic{qb},Dis{qb},cp{qb},toln,Maxp,eex,icase);
   cp{qb}=c;
  end
  Re{qb}=f{qb}-hout;
  h{qb}=hout';
  nore(qb)=norm(Re{qb})^2*(delta); %residual norm of block qp
  max_err=sum(nore);
%  cc{qb}=abs(Dx'*Re{qb});  %change from 2D
%  [max_c,b2]=max(cc{qb});
%   [max_c,q,cscra]=fast_chose_trig_atom(Re{qb}',Nx,eex,icase);  
   [max_c,q,cscra]=AtSelFFT_1(Re{qb},Nx,eex,icase);
  qq{qb}=q;  %change from 2D
  Cp(qb)=max_c;
  q(1)=qq{qb}(1);
%=================adding the Psi thing==============================
  Nat=Nat+1;
  max_err=sum(nore);
  %stopping criterion (distance)
  if (tol~= 0) & (max_err < tol) break;end;
end
max_err
%
%Copyright (C) 2016  Laura REBOLLO-NEIRA, Mathematics Department, Aston University
%
%This program is free software; you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation; either 
%version 2 of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
%without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with this program;
%if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%Boston, MA  02110-1301, USA.
