function [h,c,kap]=ProjMPTrgFFT(h,Re,LN,Di,Dic,Dis,c,toln,Max,Aux,icase)
%
% Orthogoanl projection of  Matching Pursuit  for Trigonometric dictionaries via FFT
%
% It uses the MP method for projecting the MP solution onto the spann of selected atoms
% 
%
%
% Input:
%   h       Approximation by MP
%   Re      Residual of MP
%
%   LN=[L N]  L  residual size and N dictionary cardinality
%
%   Di      Sub-Dictionary selected by MP
%   Dc      indices in Cos dict. for case 4
%   Ds      indices in Sin dict. for case 4
%   toln   numerical tolerance for the projection  
%   Max    maximum number of iteration
%   Aux    for calculating inner products via FFT
%
% Outputs:
%   h     projected approximation  
%   c     updated coefficients in the decomposition
%   kap   number of iterations for the projections
%
%
name='ProjMPTrgFFT';
L=LN(1);
tt=1:L;
N=LN(2);
a=N/L;
if icase==4 a=a/2;end
delta=1;
tol2=1e-11;
%Iniciate the  iterations
kap=0;
for it=1:Max;
%      
      [max_c,q,cscra]=AtReSelFFT(Re,N,Aux,Di,Dic,Dis,icase);
%
      if max_c < tol2 
      fprintf('%s stopped, max(|<f,D>|)<= tol2=%g.\n',name,tol2); break;
      end
      AddAt=TrgAt(q,tt,a,icase);
%      h_new=cscra*V(:,q)';
      h_new=cscra*AddAt';
      c(q)=c(q)+cscra;
      h=h+h_new;
      Re=Re-h_new;%computes the new residual 
      nor_new=norm(Re); 
      kap=kap+1;
      if  (norm(h_new)*sqrt(delta) <= toln) break;end;
end
if it==Max fprintf('%s stopped, maximum number of iterations has been reached,\n',name);
end
%
%===Laura Rebollo-Neira, 2015==============
%Mathematics  Department, Aston Uni, UK
%=========================================
