function [h, Di,cp,kap]= SPMPTrgFFT(f,LN,tol,No,toln,lstep,Max,Maxp,indx,eex,icase)
%
%SPMP Self Projected Matching Pursuit with Trigonometric dictionaries via FFT)
%It does not need dictionary LN=[L,N] (L data dimention, N number of atoms in the dictionary)
%
%
% It projects MP approximation using 
% Projected  Matching Pursuit FFT (ProjMP_FFT) every step
% For lstep=1 it converges to Orthogonal Matching Pursuit (OMP) 
% For lstep=-1 it gives plain MP, with no projection at all.
% 
% Inputs:
%   f     1D signal (Dimention N)
%   [L N] Dimention of dictionary atoms and number of atoms 
%   tol   tolerance for the approximation 
%   No    Max number of atoms in the approximation
%   toln  numerical tolerance for the ortho projection 
%   lstep  -1 for MP and 1 for Self Projections
%   Max   maximum number of iterations 
%   Maxp  maximum number of iterations for the projection step 
%   ind   set  ind=[];
%
%  Outputs:
%
%   h    Approximation of f
%   Di   indices of the selected (distinct) atoms w.r.t the original dictionary
%   c    coefficeints of the atomic decomposition (corresponding to the atoms with indices Di)
%   kap number of iterations to achieve the projection if lstep=1
%
%
L=LN(1);
N=LN(2);
a=N/L;
if icase==4 a=a/2;end
%
name='SPMPTrgFFT';
cp=zeros(1,N);
cc=zeros(1,N);
Di=[];
Dic=[];
Dis=[];
kap=[];
VR=[];
Dix=1:N;
tt=1:L;
numat=0;
numind=numel(indx);
h=0;
Re=f;
Ck=2*N/L*log2(2*N);
delta=1;
tol2=1e-11;
imp=0;
   if (lstep == -1) imp=1; end
   if (imp==1) lstep=0;end
   if (lstep == 0) Maxit2=1; lstep=Max;
   else Maxit2=Max/lstep; end
for it=1:Maxit2;
  for s=1:lstep;
   if (numat+1)<=numind
        [testx,qx]=ismember(indx(numat+1),Dix);
	if testx ~=1  error('Demanded index  %d is out of dictionary',indx(numat+1));end
       q=indx(numat+1);
       AddAt=TrAtom(q,tt,a,icase);
       V=[V AddAt]
       cc(q)=Re*AddAt';
   else
       [max_c,q,cscra]=AtSelFFT(Re,N,eex,icase);
      if max_c < tol2
        fprintf('%s stopped, max(|<f,D>|)<= tol2=%g.\n',name,tol2); break;
      end
   end
    [testq]=ismember(q,Di);
     if testq==0
        numat=numat+1; %counts the number of distinct atoms
        Di=[Di q];
         if icase==4
            if q<=N/2 Dic=[Dic q];
                else Dis=[Dis (q-N/2)];
            end
         end
     end
      AddAt=TrgAt(q,tt,a,icase);
      h_new=cscra*AddAt';
      h=h+h_new; %neeede below
      cp(q)=cp(q)+cscra;
      Re=Re-h_new;%computes the new residual 
      nor_new=norm(Re); 
      if (numat>=No |norm(Re)*sqrt(delta) < tol) break;end;
  end 
 Re=f-h;
 if imp ~= 1
if numel(Di) <= Ck
  VR=[VR AddAt];
  c=cp(Di);
 [h,c,ka]=ProjMPk(h,Re,VR,c,toln,Maxp);
% [h,c]=ProjMP_Mex(Re,h,VR,c,toln,Maxp); %swap Re h in MEX file
  cp(Di)=c;
else
%
 [h,c,ka]=ProjMPTrgFFT(h,Re,LN,Di,Dic,Dis,cp,toln,Maxp,eex,icase);
 cp=c;
%
end
 Re=f-h;
 kap=[kap ka];
end
  if (numat>=No | norm(Re)*sqrt(delta) < tol) break;end;
end
if (lstep ~=Max) & (it==Maxit2) fprintf('%s stopped, maximum number of iterations has been reached,\n',name); end
%
%=============================================
%      Laura Rebollo-Neira 2015
%  Mathematics Department, Aston Uni, UK
%=============================================
